package cn.com.victorysoft.business.indicator.dao;

import cn.com.victorysoft.business.indicator.entity.MoreSummary;
import cn.com.victorysoft.business.indicator.entity.SectionRateSummary;
import cn.com.victorysoft.business.sys.param.UnitMonthParam;
import cn.com.victorysoft.business.sys.param.UnitMonthRangeParam;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;

import java.util.List;

@Mapper
public interface MoreSummaryDao {
    int deleteByPrimaryKey(MoreSummary key);

    int insert(MoreSummary record);

    MoreSummary selectByPrimaryKey(MoreSummary key);

    int updateByPrimaryKeySelective(MoreSummary record);

    int updateByPrimaryKey(MoreSummary record);

    List<MoreSummary> selectUnit(UnitMonthParam param);

    List<MoreSummary> selectChildren(UnitMonthParam param);

    List<MoreSummary> selectRange(UnitMonthRangeParam param);

    int deleteByMonth(String month);

    /**
     * 汇总站级数据-本月与上月
     */
    void summaryStation(@Param("month") String month, @Param("prevMonth") String prevMonth);

    /**
     * 汇总站级数据-新增
     */
    void summaryStationIncrease(@Param("month") String month, @Param("prevMonth") String prevMonth);

    /**
     * 汇总站级数据-治理
     */
    void summaryStationReduce(@Param("month") String month, @Param("prevMonth") String prevMonth);
    /**
     * 汇总区级数据
     */
    void summaryArea(String month);
    /**
     * 汇总厂级数据
     */
    void summaryCompany(String month);
    /**
     * 汇总局级数据
     */
    void summaryGroup(String month);
}